/* Initial beliefs */
//pc_size(50).					// Default size of the Program Commitee (extra reviewers could be added to review submitted papers in the previous conditions) 
//n_reviewers_updated(10).		// Number of reviewers updated each year
//reviews_x_paper(3).			// Number of reviewers for each paper
//papers_x_reviewer(3).			// Number of paper reviewed by each reviewer
divergence1(1/3).				// 1st percentage of accepted papers to compute divergence between high-score papers and high-reviewed papers
divergence2(2/3).				// 1st percentage of accepted papers to compute divergence between high-score papers and high-reviewed papers
success(0).

/* Plans */
// Init step
+init
   <- !init.
   
// Generic set-up plan
+!generic_init : accept_value(AcceptanceValue) & celebration_month(Month)
   <- .print("Initiating conference [Month=",Month,",AcceptanceValue=",AcceptanceValue,"]...");
	  start.	  

// Generic end plan
+end
   <- end.
	  
// Celebrate a new edition of the conference	
+date(Month, Year, c) : celebration_month(Month) & not celebrating(_)
   <- //.println("Starting conference ", Year);      
      !updateReviewers;
	  !selectPC;
	  !callForPapers(Year);
	  +celebrating(Year).
+date(Month,_, c) : celebration_month(Month) & celebrating(CYear)
   <- .println("ERROR: Not enough time to finish conference in ", CYear).
+date(Month,_, c) : celebration_month(CMonth) & Month\==CMonth & celebrating(CYear) & 
                    periods_x_year(PeriodsXYear) & Month==(CMonth+1) mod PeriodsXYear 
   <- //.println("Submission deadline ", CYear);
	  -celebrating(CYear);
      +deadline(CYear);
      !adjustPC(CYear);
      !reviewProcess(CYear);
	  -deadline(CYear);
	  !logResults(CYear, AvgQuality, StdQuality, NSubmitted);
	  !updateConference(AvgQuality, StdQuality, NSubmitted).
+date(_,_,s)
   <- none.	  

// Update the reviewers of the conference
+!updateReviewers : n_reviewers_updated(NRUpdated)
   <- //.println("Updating the reviewers...");
	  // Discard extra reviewers
	  .findall(RNExtra, reviewer(RNExtra,extra,_), LRNExtra);
   	  .abolish(reviewer(_,extra,_));
	  .send(LRNExtra, achieve, pc_notification(out));
	  // Discard previously bad reviewers
	  for ( .range(I,1,NRUpdated) )
	  {
	    .findall(R, reviewer(R,pc,_), PC);
		if (.length(PC) > 0) 
		{
          .nth(math.floor(tools.random(nextDouble) * .length(PC)), PC, ReviewerName);
		  .abolish(reviewer(ReviewerName,_,_));
		  +discarded(ReviewerName);
		  .send(ReviewerName, achieve, pc_notification(out));
		}
	  }
	  // Initialize remaining reviewers
	  for ( reviewer(RN,_,_) )
	  {
	    .abolish(reviewer(RN,pc,_));
	    +reviewer(RN,pc,0);
	  }.
	  //.println("Updating the reviewers...[Finished]").  

// Select the Program Commitee (full set of scientists)
+!selectPC : pc_size(NReviewers) & n_scientists(NReviewers)
   <- //.println("Selecting the PC reviewers... (all)");
      while ( .count(reviewer(_,_,_),NR) & NR<NReviewers ) 
	  {  
        for ( .range(ReviewerId,1,NReviewers) ) 
	    {  
	      .concat("scientist", ReviewerId, ReviewerStrName);
		  .term2string(ReviewerName,ReviewerStrName);
		  +reviewer(ReviewerName,pc,0);
		  .send(ReviewerName, achieve, pc_notification(in));
		  if ( not image(_,_,ReviewerName,_,_) )
		  {
			  .send(ReviewerName, askOne, reviewer_skill(S), reviewer_skill(S));
			  .send(ReviewerName, askOne, type(T), type(T));
		      +image(0,0,ReviewerName,S,T);
		  }
	    }
      }.
       //.println("Selecting the PC reviewers...[Finished]");
	   
// Select the Program Commitee (partial set of scientists)
+!selectPC : pc_size(NReviewers) & n_scientists(NSc) & (NReviewers \== NSc)
   <- //.println("Selecting the PC reviewers...");
      while ( .count(reviewer(_,_,_),NR) & NR<NReviewers ) 
	  {  
        ReviewerId = math.floor(tools.random(nextDouble) * NSc) + 1;
	    .concat("scientist", ReviewerId, ReviewerStrName);
		.term2string(ReviewerName,ReviewerStrName);
	    if (not reviewer(ReviewerName,_,_) & not discarded(ReviewerName))
	    { 
		  +reviewer(ReviewerName,pc,0);
		  .send(ReviewerName, achieve, pc_notification(in));
		  if ( not image(_,_,ReviewerName,_,_) )
		  {
			  .send(ReviewerName, askOne, reviewer_skill(S), reviewer_skill(S));
			  .send(ReviewerName, askOne, type(T), type(T));
		      +image(0,0,ReviewerName,S,T);
		  }
	    }
      }
	  //.println("Selecting the PC reviewers...[Finished]");
	  .abolish(discarded(_)).
	  
// Call for papers
+!callForPapers(Year) : accept_value(AcceptValue)
   <- //.println("Sending call for papers...");
      .findall(Name, .all_names(L) & .member(Name,L) & .substring(scientist,Name), LSc);
      .send(LSc, achieve, callForPaper(Year,AcceptValue)).
	  //.println("Sending call for papers...[Finished]").

// Discard papers received after the deadline
+paper_submitted(Year,AuthorName,PaperId,_,_,_,_) : deadline(Year)
   <- .println("Warning: Out of time submission (", Year, "): ", AuthorName, " - ", PaperId);
      .abolish(paper_submitted(Year,AuthorName,PaperId,_,_,_,_));
      .send(AuthorName, achieve, notification(PaperId, out_of_time)).
	  
// Adjust the size of the Program Commitee to the number of papers received (No more scientists available) 
+!adjustPC(Year) : pc_size(PCSize) & n_scientists(PCSize) & papers_x_reviewer(InitialPxR)
   <- +actual_papers_x_reviewer(InitialPxR).
// Adjust the size of the Program Commitee to the number of papers received 
+!adjustPC(Year) : pc_size(PCSize) & n_scientists(NSc) & (PCSize \== NSc) &
                   reviews_x_paper(RxP) & papers_x_reviewer(InitialPxR) 
   <- //.println("Adjusting the PC reviewers...");
      .count(paper_submitted(Year,_,_,_,_,_,_), NP);
	  if ( math.ceil(NP*RxP/InitialPxR)>NSc )
	  {
	    ActualPxR = math.ceil(NP*RxP/NSc);
	    .println("Warning: Increasing papers per reviewer up to ", ActualPxR, " when adjusting the PC (NPapersSubmitted=",NP,")");
	  }
	  else
	  {
	    ActualPxR = InitialPxR;
	  }
      +actual_papers_x_reviewer(ActualPxR);
	  NReviewers = math.ceil(NP*RxP/ActualPxR);
	  // Use fast non-guaranteed reviewer search when there are more than 10% of free reviewers
      while ( .count(reviewer(_,_,_),NR) & NR<NReviewers & (NSc-NR)/NSc>0.1 ) 
      {  
        ReviewerId = math.floor(tools.random(nextDouble) * NSc) + 1;
	    .concat("scientist", ReviewerId, ReviewerStrName);
		.term2string(ReviewerName,ReviewerStrName);
	    if (not reviewer(ReviewerName,_,_))
	    { 
		  +reviewer(ReviewerName,extra,0);
		  .send(ReviewerName, achieve, pc_notification(in));
		  if ( not image(_,_,ReviewerName,_,_) )
		  {
			  .send(ReviewerName, askOne, reviewer_skill(S), reviewer_skill(S));
			  .send(ReviewerName, askOne, type(T), type(T));
		      +image(0,0,ReviewerName,S,T);
		  }
	    }
	  }
	  if ( NR<NReviewers)
	  {
	    // Use slow guaranteed reviewer search when there are less than 10% of free reviewers
	    .findall(Name, .all_names(L) & .member(Name,L) & not reviewer(Name,_,_) & .substring(scientist,Name), LRev);
        for ( .range(I, 0, math.min((NReviewers-NR)-1, .length(LRev)-1)) ) 
	    {  
		  .nth(I, LRev, RName);
 	      +reviewer(RName,extra,0);
		  .send(RName, achieve, pc_notification(in));
		  if ( not image(_,_,RName,_,_) )
		  {
			  .send(RName, askOne, reviewer_skill(Sk), reviewer_skill(Sk));
			  .send(RName, askOne, type(Ty), type(Ty));
		      +image(0,0,RName,Sk,Ty);
		  }
        }
	  }.
	  //.println("Adjusting the PC reviewers...[Finished]");

// Perform the reviewing process	  
+!reviewProcess(Year) 
   : reviews_x_paper(RxP) & max_value(MaxValue) & accept_value(AcceptValue) & req_quality(ReqQuality)
   <- //.println("Reviewing process...");
      -+fail_accept(0);
      -+fail_reject(0);
	  for ( .range(V, 1, MaxValue) )
	  {
	    +papers_x_value(V, 0);
	  }  
      for ( paper_submitted(Year,AuthorName,PaperId,PaperValue,AuthorAvgQ,AuthorRS,AuthorAS) )
	  {
	    // Update distribution of submitted papers
	    ?papers_x_value(PaperValue,NPV);
	    .abolish(papers_x_value(PaperValue,_));
	    +papers_x_value(PaperValue,NPV+1);

	    // Ask for reviews
	    while ( .count(review(PaperId,AuthorName,_,_),NR) & NR<RxP & actual_papers_x_reviewer(PxR) )
		{
		  .findall(R, reviewer(R,_,N) & AuthorName\==R & not review(PaperId,_,R,_) & N<PxR , AvailableReviewers);
		  if ( AvailableReviewers \== [] )
		  {		  
		    .nth(math.floor(tools.random(nextDouble) * .length(AvailableReviewers)), AvailableReviewers, ReviewerName);
		    .send(ReviewerName, askOne, review(AuthorName,PaperValue,AcceptValue,ReqQuality,Review), review(AuthorName,PaperValue,AcceptValue,ReqQuality,Review));
		    +review(PaperId,AuthorName,ReviewerName,Review);
		    ?reviewer(ReviewerName,Type,NP);
		    .abolish(reviewer(ReviewerName,_,_));
		    +reviewer(ReviewerName,Type,NP+1);
		  }
		  else
		  {
            -+actual_papers_x_reviewer(PxR+1);
	        .println("Warning: Increasing papers per reviewer up to ", PxR+1, " during the reviewing process");
		  }
		}
		// Evaluate the paper
		.findall(Review, review(PaperId,AuthorName,_,Review), Reviews);
		AvgReview = math.average(Reviews);
		if ( AvgReview == AcceptValue )
		{
		  if ( tools.random(nextDouble) < 0.5 )
		  {
		    +accepted_paper(PaperId, AuthorName, PaperValue, AvgReview, AuthorAvgQ, AuthorRS, AuthorAS);
		  }
		  else 
	      {
		    +rejected_paper(PaperId, AuthorName, PaperValue, AvgReview, AuthorAvgQ, AuthorRS, AuthorAS);
	      }		
		}
		else
		{
		  if ( AvgReview > AcceptValue )
		  {
		    +accepted_paper(PaperId, AuthorName, PaperValue, AvgReview, AuthorAvgQ, AuthorRS, AuthorAS);
		  }
		  else 
	      {
		    +rejected_paper(PaperId, AuthorName, PaperValue, AvgReview, AuthorAvgQ, AuthorRS, AuthorAS);
	      }
		}
		.abolish(review(PaperId,AuthorName,_,_));
	  }
	  // Send acceptations and compute fail_accepts
	  for ( accepted_paper(PIdAcc,AuthNameAcc,PValueAcc,_,_,_,_) )
	  {
	    .send(AuthNameAcc, achieve, notification(PIdAcc, accept));
		if (PValueAcc < AcceptValue)
		{
		  ?fail_accept(FA);
		  -+fail_accept(FA+1);
		}
	  }
	  // Send rejections and compute fail_rejects
	  for ( rejected_paper(PIdRej,AuthNameRej,PValueRej,_,_,_,_) )
	  {
	    .send(AuthNameRej, achieve, notification(PIdRej, reject));
		if (PValueRej >= AcceptValue)
		{
		  ?fail_reject(FR);
		  -+fail_reject(FR+1);
		}
	  }.
      //.println("Reviewing process...[Finished]").
	  
// Log the results of the conference	  
+!logResults(Year, AvgQuality, StdQuality, NSubmitted) 
   : .my_name(ConferenceName) & celebration_month(CMonth) & divergence1(D1) & divergence2(D2) &
     accept_value(AcceptValue) & fail_accept(FailAccept) & fail_reject(FailReject) &
	 actual_papers_x_reviewer(ActualPxR) & success(Success) & req_quality(ReqQuality)
   <- // Submitted papers
      .count(paper_submitted(Year,_,_,_,_,_,_), NSubmitted);
      // Accepted papers
	  .findall(Value, accepted_paper(_,_,Value,_,_,_,_), Values);
	  .length(Values, NAccepted);
	  if (NAccepted \== 0)
	  {
	    AvgQuality = math.average(Values);
	    if (NAccepted == 1)
	    {
		  StdQuality = 0;
	    }
	    else
	    {
		  StdQuality = math.std_dev(Values);
	    }

	    !computeDivergence(math.floor(D1*NAccepted), Divergence1);
	    !computeDivergence(math.floor(D2*NAccepted), Divergence2);
		
	    // Authors avg quality of accepted papers
	    .findall(AuthorAvgQ, accepted_paper(_,_,_,_,AuthorAvgQ,_,_), LAuthorAvgQ);
        AvgLAuthorAvgQ = math.average(LAuthorAvgQ);
	    StdLAuthorAvgQ = math.std_dev(LAuthorAvgQ);
	    .findall(AuthorAS, accepted_paper(_,_,_,_,_,_,AuthorAS), LAuthorAS);
        AvgLAuthorAS = math.average(LAuthorAS);
	    StdLAuthorAS = math.std_dev(LAuthorAS);
	    .findall(AuthorRS, accepted_paper(_,_,_,_,_,AuthorRS,_), LAuthorRS);
        AvgLAuthorRS = math.average(LAuthorRS);
	    StdLAuthorRS = math.std_dev(LAuthorRS);
	  }
	  else
	  {
	    AvgQuality = 0;
		StdQuality = 0;
		Divergence1 = 0;
		Divergence2 = 0;
	    AvgLAuthorAvgQ = 0;
		StdLAuthorAvgQ = 0;
        AvgLAuthorAS = 0;
	    StdLAuthorAS = 0;
        AvgLAuthorRS = 0;
	    StdLAuthorRS = 0;
	  }
	  // Reviewers skill
	  .findall(Skill, reviewer(R,_,_) & image(_,_,R,Skill,_), LSkills);
	  // Types of scientists in PC 
	  .count(reviewer(RN,_,_), PCSize);
	  .count(reviewer(RN,pc,_) & image(_,_,RN,_,sc), Csc);
	  .count(reviewer(RN,pc,_) & image(_,_,RN,_,r_sc), Crsc);
	  // Distribution of received and accepted papers
	  .findall(papers_x_value(V,NRcv,NAcc), papers_x_value(V,NRcv) & .count(accepted_paper(_,_,V,_,_,_,_), NAcc), PaperDist);
	  .sort(PaperDist, OrdPaperDist);
	  // Send results
	  .println("Ending conference ", Year);
      log_conference(ConferenceName, CMonth, Year, AcceptValue, Success, NSubmitted, NAccepted, 
	                 ReqQuality, AvgQuality, StdQuality, 
				     math.average(LSkills), math.std_dev(LSkills),
					 FailAccept, FailReject, OrdPaperDist, Divergence1, Divergence2, 
					 AvgLAuthorAvgQ, StdLAuthorAvgQ,
					 AvgLAuthorAS, StdLAuthorAS,
					 AvgLAuthorRS, StdLAuthorRS,
					 PCSize, ActualPxR,
					 Csc, Crsc);
	  // Clean 
      .abolish(actual_papers_x_reviewer(_));
      .abolish(papers_x_value(_,_));
	  .abolish(paper_submitted(_,_,_,_,_,_,_));
	  .abolish(accepted_paper(_,_,_,_,_,_,_));
	  .abolish(rejected_paper(_,_,_,_,_,_,_)).

// Compute paper divergence	  
+!computeDivergence(NPapers, Divergence)
   <- .findall(paper(V, Id, A), accepted_paper(Id,A,V,_,_,_,_), PaperAcceptedValues);
	  .sort(PaperAcceptedValues, PaperAcceptedValuesOrd);
	  !extractHeadPapers(NPapers, PaperAcceptedValuesOrd, [], IdAcceptedValue);
	  .findall(paper(AvgReview, Id, A), accepted_paper(Id,A,_,AvgReview,_,_,_), PaperAcceptedReviews);
	  .sort(PaperAcceptedReviews, PaperAcceptedReviewsOrd);
	  !extractHeadPapers(NPapers, PaperAcceptedReviewsOrd, [], IdAcceptedReview);
	  .intersection(IdAcceptedValue, IdAcceptedReview, IdIntersec);
	  //.println("Difference between:");
	  //.println("	Top-10-Value:", IdAcceptedValue);
	  //.println("	Top-10-Review:", IdAcceptedReview);
	  //.println("	Divergence: ", (NPapers - .length(IdIntersec)) / NPapers);
	  Divergence = (NPapers - .length(IdIntersec)) / NPapers.
+!extractHeadPapers(0,_,L,L).
+!extractHeadPapers(NPapers, [paper(_,Id,A)|L], LAux, Res): NPapers > 0
   <- !extractHeadPapers(NPapers-1, L, [paper(Id,A)|LAux], Res). 

