/* Initial beliefs */
//min_productivity(10).			// Minimum number of papers produced during one year
//max_productivity(10).			// Maximum number of papers produced during one year
//productivity_beta(1,1).		// Parameters of the Beta distribution for productivity (papers/year)
//avg_quality_beta(1,1).		// Parameters of the Beta distribution for the average quality of papers
//author_skill_beta(1,1).		// Parameters of the Beta distribution for the author skill
//reviewer_skill_beta(1,1).		// Parameters of the Beta distribution for the reviewer skill
//min_distance(3).				// Minimum optimism/pesimism when submitting papers
//max_distance(3).				// Maximum optimism/pesimism when submitting papers
//max_distance_beta(1,1).		// Parameters of the Beta distribution for the optimism/pesimism when submitting papers
//reviews_x_paper(3).			// Number of reviewers for each paper
next_paper_id(0).				// Paper id sequence
n_accepts(0).					// Number of papers accepted
n_rejects(0).					// Number of papers rejected
pc_memberships(0).				// Number of PC memberships
 
	  
/* Plans */
// Init step
+init
   <- !init.

// Generic set-up plan
+!generic_init : productivity(PapersXYear) & periods_x_year(PeriodsXYear)
   <- +prob_write(PapersXYear/PeriodsXYear);
      +max_distance_to_conference(10);
	  start. 

// Generic end plan
+end : type(Type) & .my_name(MyName)
   <- ?author_skill(AuthorSkill);
	  ?reviewer_skill(ReviewerSkill);
	  ?author_avg_quality(AvgQ);
	  ?max_distance_to_conference(MaxDist2Conf);
	  ?productivity(Productivity);
      ?next_paper_id(NPapers);
      ?n_accepts(NA);
      ?n_rejects(NR);
	  ?pc_memberships(NPCs);
      log_scientist(MyName, Type, AuthorSkill, ReviewerSkill, AvgQ, MaxDist2Conf, Productivity, NPapers, NA, NR, NPCs);
	  end.
	  
// Creation of papers
+date(_,_,s) : prob_write(Productivity) &
                      next_paper_id(PaperId) & author_skill(AS) & reviewer_skill(RS) &
					  author_avg_quality(AvgQ) & max_value(MaxValue)
   <- if ( tools.random(nextDouble) < Productivity )
      {
        if (tools.random(nextDouble) < AS) { 
          InternalValue = AvgQ;
	    } else {
          InternalValue = math.floor(tools.random(nextDouble) * MaxValue) + 1;
	    }
        if (tools.random(nextDouble) < RS) { 
          EstimatedValue = InternalValue;
        } else {
          EstimatedValue = math.floor(tools.random(nextDouble) * MaxValue) + 1;
	    }
        +available_paper(PaperId+1, InternalValue, EstimatedValue);
	    -+next_paper_id(PaperId + 1);	      
	  }
	  done.
	  
// Answer to call for papers
@callForPaper[atomic]
+!callForPaper(Year,AcceptanceValue)[source(Conference)]
   : reviewer_skill(RS) & .my_name(MyName) & 
     max_distance_to_conference(MaxDistance) & author_avg_quality(AvgQ) & author_skill(AS)
   <- for (available_paper(PaperId, Q, EstimatedValue))
      {
		if (math.abs(EstimatedValue - AcceptanceValue) <= MaxDistance)
		{
		    .abolish(available_paper(PaperId,_,_));
			+pending_paper(PaperId,Q,EstimatedValue,Conference);
			//.print("Submitting ", Conference, " ", Year, "(", AcceptanceValue, ") <-- ", Q, " - ", EstimatedValue, " D=", MaxDistance);
		    .send(Conference, tell, paper_submitted(Year,MyName,PaperId,Q,AvgQ,RS,AS));
		}
	  }.
	  
// Notification of acceptance/rejection
@accept[atomic]
+!notification(PaperId, accept)[source(Conference)] : pending_paper(PaperId,_,_,Conference) 
   <- ?n_accepts(NA);
      -+n_accepts(NA+1);
      .abolish(pending_paper(PaperId,_,_,_)).
@reject[atomic]
+!notification(PaperId, reject)[source(Conference)] : pending_paper(PaperId,_,_,Conference) 
   <- ?n_rejects(NR);
      -+n_rejects(NR+1);
	  .abolish(pending_paper(PaperId,_,_,_)).
      //+available_paper(PaperId,Q,EstimatedValue).
@out_of_time[atomic]
+!notification(PaperId, out_of_time)[source(Conference)] : pending_paper(PaperId,Q,EstimatedValue,Conference) 
   <- .abolish(pending_paper(PaperId,_,_,_));
      +available_paper(PaperId,Q,EstimatedValue).

// Notification of PC membership
@in[atomic]
+!pc_notification(in) 
   <- ?pc_memberships(N);
      -+pc_memberships(N+1).
@out[atomic]
+!pc_notification(out) 
   <- ?pc_memberships(N);
      -+pc_memberships(N-1).

